/*
 * Bayer-mask pixel processing defines.
 *
 * Copyright (c) 2004 Steve Hosgood <steve@caederus.com> Jul-Nov 2004
 *
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. NO WARRANTY OF ANY KIND is expressed or implied.
 */


# ifndef _BAYER_H_
# define _BAYER_H_

# include	<linux/kernel.h>

/*
 * Once a raw-pixel image of 'height' by 'width' pixels has been received, a Bayer-mask
 * processor function (of typedef 'bayer_f') is called to convert it to a colour image in one of
 * the many FourCC formats. The two pixel-packer functions deal with the even and odd pixels - they
 * need to point to different routines for FourCC formats like "YUYV" or "VYUY". They can both
 * point to a shared routine for formats like "RGB24" or "RGB565".
 *
 * The pixel-packer functions have a typedef of 'matrix_f'. They take raw camera R',G',B' signals
 * and (if necessary) rematrix them to create Rec.709 RGB then pack them into the output image
 * according to the desired FourCC format. It is REALLY IMPORTANT that they run as fast as
 * possible as they are called for every pixel in every image.
 *
 * In the case of USB camera support (where these concepts first appeared) the Bayer
 * processor is called once per image by the code responsible for handling the camera's USB
 * bridge chip. The code for the pixel-packers are in the kernel module responsible for handling
 * the actual imager chip due to the fact that any colour rematrixing information is sensor-dependant.
 * For instance, even the apparently similar OV2610 and OV3610 devices (both from Omnivision) have
 * RGB filters with quite different colour responses - and therefore have different rematrixing
 * coefficients. 
 */


typedef unsigned char	*matrix_f(unsigned char *pOut, unsigned char rdash, unsigned char gdash, unsigned char bdash ); /* pixel-packer for the output frame's format */
typedef void		bayer_f( unsigned char *pOut, unsigned char *pIn, int width, int height, int flags, matrix_f *rematrix_even, matrix_f *rematrix_odd );

# endif /* _BAYER_H_ */

